package com.zcxk.rmcp.core.dao;

import com.zcxk.rmcp.api.dto.alarm.AlarmTypeDto;
import com.zcxk.rmcp.api.dto.alarm.AlarmTypePageDto;
import com.zcxk.rmcp.core.entity.AlarmType;
import org.apache.ibatis.annotations.Param;


import java.util.List;

import org.apache.ibatis.annotations.Mapper;

/**
 * 设备告警类型(AlarmType)表数据库访问层
 *
 * @author hym
 * @since 2021-07-30 10:57:47
 */
@Mapper
public interface AlarmTypeMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    AlarmType selectById(Integer id);


    /**
     * 查询全部
     *
     * @return 对象列表
     */
    List<AlarmType> selectAll();

    /**
     * 通过实体作为筛选条件查询
     *
     * @param alarmType 实例对象
     * @return 对象列表
     */
    List<AlarmTypeDto> selectList(AlarmTypePageDto alarmType);

    /**
     * 新增数据
     *
     * @param alarmType 实例对象
     * @return 影响行数
     */
    int insert(AlarmType alarmType);

    /**
     * 批量新增
     *
     * @param alarmTypes 实例对象的集合
     * @return 影响行数
     */
    int batchInsert(@Param("alarmTypes") List<AlarmType> alarmTypes);

    /**
     * 修改数据
     *
     * @param alarmType 实例对象
     * @return 影响行数
     */
    int update(AlarmType alarmType);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 影响行数
     */
    int deleteById(Integer id);

    /**
     * 查询总数据数
     *
     * @return 数据总数
     */
    int count();


    /**查询告警类型所拥有的告警规则数目
      * @param ids
     * @description
     * @author hym
     * @updateTime 2021/8/3 9:45
     * @return java.util.List<com.zcxk.rmcp.api.dto.alarm.AlarmTypeDto>
     * @throws
     */
    List<AlarmTypeDto> selectTypesWithRules(List<Integer> ids);
    /**删除告警类型
      * @param ids
     * @description
     * @author hym
     * @updateTime 2021/8/3 9:56
     * @return void
     * @throws
     */
    void delete(List<Integer> ids);
}
